<?php

/* Always have the fallback driver loaded */
require_once ('op5lic_drv_unlicensed.php');

/**
 * License library for op5 products
 */
class op5lic {
	/*
	 * fetch_metric returns this for infinit value. Always use === or !== to
	 * compare
	 */
	const METRIC_INFINITY = PHP_INT_MAX;
	const DEFAULT_LICENSE_FILE = '/etc/op5license/op5license.xml';
	const OP5_LICENSE_URL = 'https://www.op5.com/software-license';
	const OP5_CONTACT_URL = 'http://www.op5.com/about/contact-us';
	const OP5_BLAFFA_URL = '/monitor/index.php/license'; // In the license, we
	                                                     // can assume that ninja
	                                                     // is installed here...
	const OP5_SALES_EMAIL = 'cr@op5.com';
	const OP5_SALES_PHONE_NUMBER = '+46 (0)8 58 83 01 00';
	private $driver = false;

	/**
	 * Constructor
	 *
	 * @param $file string
	 *        	optional parameter. Filename of licensefile
	 * @return void
	 */
	public function __construct($file = self::DEFAULT_LICENSE_FILE) {
		$drivers = array ('op5xml','op5v1','op5v2','trial');
		try {
			foreach ($drivers as $drv) {
				try {
					$class = "op5lic_drv_$drv";
					require_once ($class . '.php');
					$this->driver = new $class($file);
					return;
				} catch (op5lic_skip_exception $e) {
					/* Well, don't do anything */
				}
			}
			throw new op5lic_invalid_exception(
				'No matching license driver found');
		} catch (op5lic_invalid_exception $e) {
			$this->driver = new op5lic_drv_unlicensed($e);
		}
	}
	public function fetch_property($prop) {
		return $this->driver->fetch_property($prop);
	}

	/**
	 * Queries the license if the specified usage is ok according to the
	 * license.
	 *
	 * This method does not fetch any information about the system, so
	 * everything needs to be provided through the metric-array (with one
	 * exception, see below)
	 *
	 * The metrics array should contain a list of metrics, and their usage, for
	 * example, a system with 47 monitor hosts, and two pollers should run:
	 * query(array('monitor' => 47, 'pollers' => 2)).
	 *
	 * The exception is license expiration time, which is the metric
	 * 'time_expire' as unix timestamp. If not specified, the query-method fills
	 * in that field before running the test with the system time. To explicitly
	 * skip expiration time check, specify the metric 'time_expire' => false
	 *
	 * @param $metrics array
	 *        	Metrics to tests
	 * @param $message array
	 *        	reference variable that will be set to an array of status
	 *        	messages. Optional parameter.
	 * @return boolean
	 */
	public function query(array $metrics, &$message = false) {
		$valid = true;

		/* Handle all keys as lowercase, to make it insensitive */
		$metrics = array_change_key_case($metrics, CASE_LOWER);

		if (!array_key_exists('time_expire', $metrics)) {
			/* If time isn't specified, assume time is right now */
			$metrics['time_expire'] = time();
		} else {
			/* Make it possible to explicitly ignore time */
			if ($metrics['time_expire'] === false) {
				unset($metrics['time_expire']);
			}
		}

		$result = $this->driver->query($metrics);
		$message = array ();
		if (isset($result['messages']['load']))
			$message = $result['messages']['load'];

		$valid = true;
		foreach ($metrics as $metric => $usage) {
			if ($usage == 0)
				continue;
			$size = 0;
			if (isset($result['metrics'][$metric]))
				$size = $result['metrics'][$metric];
			if ($size < $usage) {
				$valid = false;
			}
		}
		return $valid;
	}

	/**
	 * Takes a metric, a usage amount, and a threshold as a factor of the
	 * license size.
	 *
	 * Return true if theshold isn't reached.
	 *
	 * Example:
	 * $threshold_reached = query_threshold( 'monitor', 12, 0.9 );
	 *
	 * @param
	 *        	$metric
	 * @param
	 *        	$usage
	 * @param
	 *        	$threshold
	 * @param $message array
	 *        	containing the error messages
	 * @return boolean
	 */
	public function query_threshold($metric, $usage, $threshold,
		&$message = false) {
		$message = array ();
		$licensed = $this->fetch_metric($metric);
		if ($licensed)
			$fraction = (float) $usage / (float) $licensed;
		else
			$fraction = 0;

		if ($licensed === self::METRIC_INFINITY) {
			$message[] = sprintf("No limitation in license size for %s",
				$metric);
			return true;
		}
		if ($licensed == 0 && $usage > 0) {
			$message[] = sprintf(
				"You are not licensed to have any %s, you use %d", $metric,
				$usage);
		} else if ($usage > $licensed) {
			$message[] = sprintf(
				"You are exceeding your license (you use %d, you may use %d) for %s",
				$usage, $licensed, $metric);
		} else if ($fraction >= $threshold) {
			$message[] = sprintf(
				"You are using %0.1f%% (%d out of %d) of your license for %s",
				$fraction * 100, $usage, $licensed, $metric);
		}
		if ($message)
			return false;
		$message[] = sprintf("You are using %0.1f%% of your license for %s",
			$fraction * 100, $metric);
		return true;
	}

	/**
	 * Given a metric, this method returns the number of objects allowed for
	 * that specific metric.
	 *
	 * A metric is something measurable in the system. Some are:
	 * - 'monitor' => number of hosts allowed for monitor
	 * - 'logserver' => number of hosts allowed for logserver
	 * - 'pollers' => how many pollers the system is allowed to handle
	 * - and many more, which is up to the license file to define.
	 *
	 * If the license doesn't define the metric, it is assumed that no objects
	 * is allowed, which means 0.
	 *
	 * If the license allowes infinit number of objects, this metric returns the
	 * magic value of self::METRIC_INFINITY. Make sure you use strict comparison
	 * of the that value, i.e. === or !==
	 *
	 * @param $product integer
	 *        	number of objects allowed for the given metric, or
	 *        	self::METRIC_INFINITY
	 */
	public function fetch_metric($metric) {
		$result = $this->driver->query(array ($metric => 0));
		if (!isset($result['metrics'][$metric]))
			return 0;
		return $result['metrics'][$metric];
	}

	/**
	 * Fetch a list of defined metrics for the license.
	 * Notable is that the license might grant access to other metrics than what
	 * is returned here, for example a trial allows full access to everything.
	 * This metod should only be used to visualize the status of the license
	 * when displaying lists of metrics, not for any license checking.
	 *
	 * For license checking, use query() instead. For statistics given a
	 * specific metric, the fetch_metric() is often better to use.
	 */
	public function fetch_license_metrics() {
		$result = $this->driver->query(array ());
		return $result['metrics'];
	}

	/**
	 * Returns information about the customer
	 */
	public function fetch_customer_info() {
		return $this->driver->fetch_customer_info();
	}

	/**
	 * "Old" API.
	 * deprecated.
	 */
	public function fetch_load_messages() {
		return $this->fetch_messages('load');
	}

	/**
	 * "Old" API.
	 * deprecated.
	 */
	public function fetch_load_errors() {
		return $this->fetch_messages('load');
	}

	/**
	 * Returna a list of messages for a given context.
	 * The result is guaranteed to be an array, but it might be empty, if not
	 * messages are generated for that context.
	 *
	 * @param $context string
	 *        	The context name
	 * @return array list of messages
	 */
	public function fetch_messages($context) {
		$result = $this->driver->query();
		if (!isset($result['messages'][$context]))
			return array ();
		return $result['messages'][$context];
	}

	/**
	 * Format time difference
	 *
	 * @param $deltat int
	 *        	number of seconds
	 * @return a string representing the interval as X days Y hours Z minutes
	 */
	static public function format_time_diff($deltat, $precision = 1) {
		$parts = array ();
		$parts['day'] = intval($deltat / (60 * 60 * 24));
		$parts['hour'] = intval($deltat / (60 * 60)) % 24;
		$parts['minute'] = intval($deltat / 60) % 60;
		$parts['second'] = intval($deltat) % 60;

		$cur_precision = 0;
		$ret = array ();
		foreach ($parts as $k => $v) {
			/* If we found a suffix, increment */
			if ($cur_precision)
				$cur_precision++;

				/* If the suffix has a value, possibly print it */
			if ($v > 0) {
				/* If we havn't already found a suffix, we have now */
				if (!$cur_precision)
					$cur_precision = 1;

					/* Only display a precision of two suffixes */
				if ($cur_precision <= $precision)
					$ret[] = $v . ' ' . $k . ($v > 1 ? 's' : '');
			}
		}

		return join(' ', $ret);
	}
}
